//+------------------------------------------------------------------+
//|                                                  MACD Alert2.mq4 |
//|                                       http://www.trading-team.it |
//+------------------------------------------------------------------+
#property copyright "Silvio Invernici"
#property link      "http://www.trading-team.it"

#property indicator_separate_window
#property indicator_buffers 6
#property indicator_color1  Silver
#property indicator_color2  Silver
#property indicator_color3  Blue
#property indicator_color4  Green
#property indicator_color5  Red
#property indicator_color6  Red
#property indicator_width1  2
#property indicator_width2  2
#property indicator_width4  4
#property indicator_width5  4
#property indicator_width6  4

extern int    FastEMA          = 5;
extern int    SlowEMA          = 10;
extern int    SignalSMA        = 5;
extern bool   alertsOn         = true;
extern bool   alertsOnCurrent  = false;
extern bool   alertsMessage    = true;
extern bool   alertsSound      = true;
extern bool   alertsNotify     = false;
extern bool   alertsEmail      = false;
extern string soundFile        = "alert2.wav";
extern bool   ShowArrows       = true;
extern string arrowsIdentifier = "macd Arrows1";
extern double arrowsUpperGap   = 0.5;
extern double arrowsLowerGap   = 0.5;
extern color  arrowsUpColor    = LimeGreen;
extern color  arrowsDnColor    = Red;
extern int    arrowsUpCode     = 225;
extern int    arrowsDnCode     = 226;
extern int    arrowsUpSize     = 2;
extern int    arrowsDnSize     = 2;

double macd[];
double sign[];
double macdDa[];
double macdDb[];
double macdUp[];
double macdDn[];
double trend[];
double cross[];



int init() 
{
   IndicatorBuffers(8);
   SetIndexBuffer(0,macdUp); SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(1,macdDn); SetIndexStyle(1,DRAW_HISTOGRAM);
   SetIndexBuffer(2,sign);
   SetIndexBuffer(3,macd);
   SetIndexBuffer(4,macdDa);
   SetIndexBuffer(5,macdDb);
   SetIndexBuffer(6,trend);
   SetIndexBuffer(7,cross);
   IndicatorShortName("MACD Alert2(" + FastEMA + "," + SlowEMA + "," + SignalSMA + ")");
   return (0);
}

int deinit()
{
   deleteArrows();
return(0); 
} 

//+----------------------start---------------------------------------+
int start() 
{
   int counted_bars=IndicatorCounted();
   int i,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = MathMin(Bars-counted_bars,Bars-1);
         
   //
   //
   //
   //
   //
   
   if (trend[limit]==-1) CleanPoint(limit,macdDa,macdDb); 
   for (i=limit; i >= 0; i--) macd[i] = iMA(NULL,0,FastEMA,0,MODE_EMA,PRICE_CLOSE,i) - iMA(NULL,0,SlowEMA,0,MODE_EMA,PRICE_CLOSE,i);
   for (i=limit; i >= 0; i--) sign[i] = iMAOnArray(macd,0, SignalSMA, 0, MODE_SMA, i);
   for (i=limit; i >= 0; i--) 
   { 
       macdDa[i] = EMPTY_VALUE;
       macdDb[i] = EMPTY_VALUE;
       macdUp[i] = EMPTY_VALUE;
       macdDn[i] = EMPTY_VALUE;
       trend[i]  = trend[i+1];
       cross[i]  = cross[i+1];
       if (macd[i]>0)       cross[i] = 1;
       if (macd[i]<0)       cross[i] =-1;
       if (macd[i]>sign[i]) trend[i] = 1;
       if (macd[i]<sign[i]) trend[i] =-1;
       if (cross[i] == 1)   macdUp[i] = macd[i];
       if (cross[i] ==-1)   macdDn[i] = macd[i];
       if (trend[i]==-1) PlotPoint(i,macdDa,macdDb,macd);
       
       //
       //
       //
       //
       //
       
       if (ShowArrows)
       {
            deleteArrow(Time[i]);
            if (trend[i]!=trend[i+1])
            {
               if (trend[i] == 1) drawArrow(i,arrowsUpColor,arrowsUpCode,arrowsUpSize,false);
               if (trend[i] ==-1) drawArrow(i,arrowsDnColor,arrowsDnCode,arrowsDnSize, true);
            }
        }
   }
   
   //
   //
   //
   //
   //
   
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1;
      if (cross[whichBar] != cross[whichBar+1])
      if (cross[whichBar] == 1)
            doAlert("crossing up");
      else  doAlert("crossing down");       
   }
return (0);
}

//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if (i>=Bars-3) return;
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (i>=Bars-2) return;
   if (first[i+1] == EMPTY_VALUE)
      if (first[i+2] == EMPTY_VALUE) 
            { first[i]  = from[i];  first[i+1]  = from[i+1]; second[i] = EMPTY_VALUE; }
      else  { second[i] =  from[i]; second[i+1] = from[i+1]; first[i]  = EMPTY_VALUE; }
   else     { first[i]  = from[i];                           second[i] = EMPTY_VALUE; }
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

void doAlert(string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[0]) {
          previousAlert  = doWhat;
          previousTime   = Time[0];

          //
          //
          //
          //
          //

          message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," Macd ",doWhat);
             if (alertsMessage) Alert(message);
             if (alertsNotify)  SendNotification(message);
             if (alertsEmail)   SendMail(StringConcatenate(Symbol()," Macd  "),message);
             if (alertsSound)   PlaySound(soundFile);
      }
}

//
//
//
//
//

void drawArrow(int i,color theColor,int theCode,int theWidth,bool tup)
{
   string name = arrowsIdentifier+":"+Time[i];
   double gap  = 3.0*iATR(NULL,0,20,i)/4.0;   
   
      //
      //
      //
      //
      //
      
      ObjectCreate(name,OBJ_ARROW,0,Time[i],0);
         ObjectSet(name,OBJPROP_ARROWCODE,theCode);
         ObjectSet(name,OBJPROP_COLOR,theColor);
         ObjectSet(name,OBJPROP_WIDTH,theWidth);
         if (tup)
               ObjectSet(name,OBJPROP_PRICE1,High[i]+ arrowsUpperGap * gap);
         else  ObjectSet(name,OBJPROP_PRICE1,Low[i] - arrowsLowerGap * gap);
}

//
//
//
//
//

void deleteArrows()
{
   string lookFor       = arrowsIdentifier+":";
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
         if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
}
void deleteArrow(datetime time)
{
   string lookFor = arrowsIdentifier+":"+time; ObjectDelete(lookFor);
}

      